
c       .................................................................
c       .  Scattering program for an inclusion imbedded inside the      .
c       .  host droplet.  The input data is taken from the file         .
c       .  "input.in" and the variables and arrays are defined in       .
c       .  the file "declare.def".                                      .
c       .................................................................
c This program was written by Dat Ngo and the derivation is similar to
c that given in his dissertation and in the article:
c Ngo, Videen, and Chylek, Comp. Phys. Comm. 1077, 94-112 (1996).
c If there are problems, report them to me at
c videen@atm.dal.ca
c

        implicit double precision (a-h,o-z)

        INCLUDE 'declare.def'

        open(unit=4,file='input.in',status='old')
        rewind(4)
        open(unit=8,file='output.out')
c       open(unit=1,file='file1.in')
c       open(unit=2,file='file2.in')

c  ..........................................................
c  .  read in the input values from "input.in"              .
c  ..........................................................
        read(4,*) wavel
        read(4,*) rad1
        read(4,*) rad2
        read(4,*) distance
        read(4,*) m_r1,m_i1
        read(4,*) m_r2,m_i2
        read(4,*) inc_ang
        read(4,*) num
        read(4,*) phi

        pi=1.0
        ccc=dcmplx(0.0,pi)
        pi=4.0*datan(pi)

      m_1=dcmplx(m_r1,m_i1)
      m_2=dcmplx(m_r2,m_i2)
      k0=2.*pi/wavel
      k1=k0*m_1
      k2=k0*m_2
      x0=k0*rad1
      x1=k1*rad1
      x2=k1*rad2
      x3=k2*rad2

c     nbg= # of iterations for the big sphere and loops
      nbg=cdabs(x0) + 4.*cdabs(x0)**.333 + 2.0
c      print*,'the value of nbg=',nbg
c      print*,'enter your value of nbg ==>'
c      read(5,*)nbg
c .......................................................................
c .     Calculate Bessel Functions for the first boundary at rad1       .
c .     We have bessel as an incident field of the sphere, and a        .
c .     hankel as the scattered field.  Their arguments are x0 and cx0  .
c .     Then we have hankel1 and hankel2 inside the sphere; and         .
c .     their argument is k1*rad1=x1.                                   .
c .......................................................................
      cx0=dreal(x0)
      call bessel(nbg,isize,besj_o,cx0)         
      call newman(nbg,isize,temp,cx0)
      call c_bessel(nbg,isize,hankel_o,x0)
      do 11 i=-1,nbg
        hankel_o(i)=hankel_o(i)+ccc*temp(i)
 11   continue                                  

      call c_bessel(nbg,isize,hankel1_1,x1)
      call c_bessel(nbg,isize,hankel2_1,x1)
      call c_newman(nbg,isize,tempc,x1)
      do 12 i=-1,nbg
        hankel1_1(i)=hankel1_1(i) + ccc*tempc(i)
        hankel2_1(i)=hankel2_1(i) - ccc*tempc(i)
 12   continue                                  

c .......................................................................
c .     Now we match boundary at rad2.  Here we have two hankels in     .
c .     the host sphere, and their argument is k1*rad2=x2               .
c .     Then we have a bessel inside the inclusion with its argument    .
c .     being x3.  Note: since this is for the small inclusion, we      .
c .     must take care to have spherical bessel functions which are     .
c .     within the limits of that size; the order should never exceed   .
c .     the size of the sphere because it'll blow up in your face       .
c .......................................................................
      nsm=cdabs(x2) + 4.*cdabs(x2)**.333 + 2.0
      if(m_r2.gt.0.0) call c_bessel(nsm,isize,besj_2,x3)        
      call c_bessel(nsm,isize,hankel1_2,x2)
      call c_bessel(nsm,isize,hankel2_2,x2)
      call c_newman(nsm,isize,tempc,x2)
      do 13 i=-1,nsm
        hankel1_2(i)=hankel1_2(i) + ccc*tempc(i)
        hankel2_2(i)=hankel2_2(i) - ccc*tempc(i)
 13   continue                          
        
c .......................................................................
c .    Calculate Riccati-Bessel Functions and Derivatives needed        .
c .     Note that zeta1_1 means the 1st zeta at k1a1                    .
c .     and       zeta2_1 means the 2nd zeta at k1a1                    .
c .     and       zeta1_2 means the 1st zeta at k1a2                    .
c .......................................................................
        psi_o(-1)=cx0*besj_o(-1)
        do 21 n=0,nbg
                psi_o(n)=cx0*besj_o(n)
                dpsi_o(n)=cx0*besj_o(n-1) -n*besj_o(n)
  21    continue
        call ricatti(nbg,hankel_o,zeta_o,dzeta_o,x0)
        call ricatti(nbg,hankel1_1,zeta1_1,dzeta1_1,x1)
        call ricatti(nbg,hankel2_1,zeta2_1,dzeta2_1,x1)
        call ricatti(nsm,hankel1_2,zeta1_2,dzeta1_2,x2)
        call ricatti(nsm,hankel2_2,zeta2_2,dzeta2_2,x2)
        call ricatti(nsm,besj_2,psi_2,dpsi_2,x3)

c .......................................................................
c .     Now we must solve for the Quality factors by using equations    .
c .     17 and 18; These Q's relate the inclusion's field coeffs        .
c .     to the outter sphere's coefficients, and it contains infos such .
c .     as the inclusions's radius and refractive index                 .
c .     Note that if the index of refraction of the inclusion is nega-  .
c .     tive, then we'll take that to mean it is a perfect conductor,   .
c .     and the simplification is in the "else" part of the if-else     .
c .     statement.  We cannot use index > 100 because it'll blow up     .
c .......................................................................
        if(m_r2.gt.0.0) then
         do 25 n=0,nsm
          fact1=k1*dzeta2_2(n)*psi_2(n)-k2*zeta2_2(n)*dpsi_2(n)
          fact2=k2*zeta1_2(n)*dpsi_2(n)-k1*dzeta1_2(n)*psi_2(n)
          Q_r(n)=fact1/fact2
          fact3=k2*dzeta2_2(n)*psi_2(n)-k1*zeta2_2(n)*dpsi_2(n)
          fact4=k1*zeta1_2(n)*dpsi_2(n)-k2*dzeta1_2(n)*psi_2(n)
          Q_s(n)=fact3/fact4
 25      continue
c  Now to fill in the rest of the array with a constant value.  This is a must.
         do 26 n=nsm+1,nbg
          Q_r(n)=dcmplx(1.0,0.0)
          Q_s(n)=Q_r(n)
 26      continue
        else
c ........................................
c . The inclusion is a perfect conductor .
c ........................................
          print*,'The inclusion is a perfect conductor'
          do 35 n=0,nsm
            Q_r(n)=-zeta2_2(n)/zeta1_2(n)
            Q_s(n)=-dzeta2_2(n)/dzeta1_2(n)
 35       continue
          do 36 n=nsm+1,nbg
            Q_r(n)=dcmplx(1.0,0.0)
            Q_s(n)=Q_r(n)
 36       continue
        endif

c .......................................................................
c .    Calculate translation coefficients when the separation of the    .
c .    center to center radii is d.  We will first find the bessel      .
c .    function which describes the separation, k1d                     .
c .......................................................................
        xd=(k0*m_1)*distance
        if (xd.eq.0.0) then
                nsm=2
        else
                nsm=cdabs(xd) + 4.*cdabs(xd)**.333 +2.
        endif   
        call c_bessel(3*nsm,isiz4,besj_d,xd)

c ...............................................
c .    using 23 and 24 from the paper first     .
c .    keeping in mind that cuplo = c0,0(np) and.
c .    cuplo1 = c-1,0(np) for starters          .
c ...............................................

      do 110 np=0,nbg*4
        fnp=dfloat(np)
        cuplo(np)=besj_d(np)*dsqrt(2.*fnp+1.)
        cuplo1(np)=-cuplo(np)
 110  continue
      do 115 np=0,nbg*2
        cuplom(0,np)=cuplo(np)          
 115  continue

c .................................................
c .     Now use equation 25, this is where we     .
c .     perform the switcharoos and recursively   .
c .     solve for the translation coeff.  The     .
c .     result is stored into the matrix cuplom   .
c .................................................
      do 140 n=1,nbg*2
        fn=dfloat(n)
        do 120 np=0,nbg*4-n
          fnp=dfloat(np)
          c=cdsqrt(dcmplx((2.*fnp+1.)/(2.*fn-3.)))*(fn-1.)
          c=cuplo1(np)*c
          cuplo1(np)=cuplo(np)
          cuplo(np)=c
 120    continue

        do 130 np=0,nbg*4-n
          fnp=dfloat(np)
          c=-(fnp+1.)*dsqrt((2.*fn-1.)/(2.*fnp+3.))*cuplo1(np+1)
       c=c+fnp*cdsqrt(dcmplx((2.*fn-1.)/(2.*fnp-1.)))*cuplo1(np-1)
          c=c+cuplo(np)
          cuplo(np)=c*dsqrt((2.*fn+1.)/(2.*fnp+1.))/fn
          if (np.le.nbg*2) then
            cuplom(n,np)=cuplo(np) 
c       write(8,*)n,np,cuplom(n,np)
          endif
 130    continue
 140  continue
 
c ...................................................
c . Now that we have the elements in the matrix     .
c . cuplom(n,np), we can start the iterations in    .
c . m to get the rest of the translation coeff      .
c . Then we'll have the complete set of C(n,m)np    .
c ...................................................
        do 299 m=0,nbg
c       do 299 m=1,1
c       ......................................................
c       . But first we need to calculate, for each m, the    .
c       . Legendre Polynomials for incident angle theta      .
c       ......................................................
            ii=nbg+1
            x=dcos(pi*inc_ang/180.)
            mmm=m+1
            call nplgndr(mmm,ii,isize1,legpol1,x)
            mmm=m
            call nplgndr(mmm,ii,isize1,legpol2,x)
            if (m.gt.0) then
               mmm=m-1
               call nplgndr(mmm,ii,isize1,legpol,x)
            else
               do 165, jjj=-2,isize1
                  legpol(jjj)=-legpol1(jjj)
 165           continue
            endif

            do 180 i=m,nbg
               fi=dfloat(i)
               tau=dsqrt((fi+m+1.)*(fi-m))*legpol1(i)
               pie=tau
               tau=(tau-dsqrt((fi+m)*(fi-m+1.))*legpol(i))/2.0
               if (dabs(x).gt.0.5) then
c       .................................................
c       . The angle theta is between 0 and 60 degrees   .
c       .................................................
                 pie=pie+dsqrt((fi+m)*(fi-m+1.))*legpol(i)
                 pie=pie*.5/x
               else
c       .................................................
c       . theta is between 60 and 90; use this so the   .
c       . Polynomial doesn't blow up at 90 or 0 degrees .
c       .................................................
                 pie=m*legpol2(i)/dsin(inc_ang*pi/180.)
               endif
               if (i.gt.0) then
                c = (ccc**i)/(fi*(fi+1.))
                ba(i)=-c*pie    
                aa(i)=c*tau     
               endif
c       if(m.eq.1) write(8,*)i,aa(i),ba(i)                      
 180        continue
c ...............................................................
c .     That ends the Calculation for the Legendre Polynomials  .
c .     for each value of m.  The incident field coefficients   .
c .     for the TE case are stored in aa(i) and ba(i); the TM   .
c .     case will be done later by using eqns 48 & 49           .
c ...............................................................

c ...............................................................
c . Now to find the rest of the matrix in C(n,m)np.  For each   .
c . value of m that is greater than 0, say 1, we can find the   .
c . matrix C(n,1)np and store that into cuplom(n,np); then      .
c . the next iteration of m, i.e. m=2, we can find C(n,2)np     .
c . by already having C(n,1)np stored in the matrix cuplom(n,np).
c . Use equation 26                                             .
c ...............................................................
      if (m.gt.0) then
        do 155, n=m-1,2*nbg-m+1
                do 154, np=m-1,2*nbg-m+1
                        cuplom1(n,np)=cuplom(n,np)
 154            continue
 155    continue

        do 160 n=m,2*nbg-m
          fn=dfloat(n)
          do 159 np=m,2*nbg-m
           fnp=dfloat(np)
       c=dsqrt((fnp-m+1.)*(fnp+m)*(2.*fnp+1.))*cuplom1(n,np)
           cuplom(n,np)=c
           c=xd*dsqrt((fnp-m+2.)*(fnp-m+1.)/(2.*fnp+3.))
           cuplom(n,np)=cuplom(n,np)-c*cuplom1(n,np+1)
           c=xd*dsqrt((fnp+m)*(fnp+m-1.)/(2.*fnp-1.))
           cuplom(n,np)=cuplom(n,np)-c*cuplom1(n,np-1)
           c=dsqrt((fn-m+1.)*(fn+m)*(2.*fnp+1.))
           cuplom(n,np)=cuplom(n,np)/c
 159      continue
 160    continue
      endif

c ..................................................
c . Cuplom(n,np) now has the most recent values    .
c . for the most recent value of m.  Note that     .
c . we didn't find C(n,0)np because we already     .
c . have those values stored in the cuplom matrix  .
c ..................................................

c ............................................................
c . Now that we have the matrix which contains the C(n,m)np  .
c . we can go ahead and find the translation coefficients    .
c . A(n,m)np and B(n,m)np by using eqns 21 & 22. Note,       .
c . we do need to store TransAm and TransBm into a matrix    .
c . like we did for the C(n,m)np for later manipulation      .
c ............................................................
      do 190 n=m,nbg
         pie=0.0
         tau=0.0
         do 195 np=m,nbg
            fnp=dfloat(np)
            tau=(fnp-m+1.0)*(fnp+m+1.0)
            tau=tau/((2.*fnp+1.)*(2.*fnp+3.))
            tau=dsqrt(tau)
            TransA=-xd/(fnp+1.0)
            TransA=TransA*tau*cuplom(n,np+1)
c           if(np.eq.nbg) TransA=0.0
            TransB=0.0
            if (np.gt.0) then
             pie=(fnp-m)*(fnp+m)
             pie=pie/((2.*fnp-1.)*(2.*fnp+1.))
             pie=dsqrt(pie)
             c=-xd*pie*cuplom(n,np-1)/fnp
             TransA=TransA+c
             TransB=-ccc*xd*m*cuplom(n,np)/(fnp*(fnp+1.))
            endif
            TransA=cuplom(n,np)+TransA
            TransAm(n,np)=TransA
            TransBm(n,np)=TransB
c       if(m.eq.1.or.m.eq.2)write(8,*)n,np,TransA,TransB
 195     continue
 190  continue

      numel=nbg-m+1
      numtot=numel+numel
      do 220 n=m,nbg
            i=n-m+1
            do 215 np=m,nbg
c ...............................................................
c . The Q's have the index of the inclusion k2 inside them      .
c . and the TransAm and TransBm have the displacement d inside  .
c . them; matrixlu therefore is the 1st to contain both pieces  .
c . of information.  The equations are 33-36                    .
c ...............................................................
              fact1=dzeta_o(n)*(zeta2_1(n)+Q_r(np)*zeta1_1(n))
              fact2=zeta_o(n)*(dzeta2_1(n)+Q_r(np)*dzeta1_1(n))
              fact3=dzeta_o(n)*(zeta2_1(n)+Q_s(np)*zeta1_1(n))
              fact4=zeta_o(n)*(dzeta2_1(n)+Q_s(np)*dzeta1_1(n))
              ii=np-m+1

c       print*,n,np
              c=TransAm(np,n)*(k0*fact1 - k1*fact2)
              matrixlu(i,ii)=c
              matrix(i,ii)=c
c       print*,c

              c=TransBm(np,n)*(k0*fact3 - k1*fact4)
              matrixlu(i,ii+numel)=c
              matrix(i,ii+numel)=c
c       print*,c

              c=TransBm(np,n)*(k1*fact1 - k0*fact2)
              matrixlu(i+numel,ii)=c
              matrix(i+numel,ii)=c
c       print*,c

              c=TransAm(np,n)*(k1*fact3 - k0*fact4)
              matrixlu(i+numel,ii+numel)=c
              matrix(i+numel,ii+numel)=c
c       print*,c
c       print*,' '
 215        continue
 220  continue

c................................................................
c. THE MATRIX IS FILLED!!  That was the heart of the problem    .
c................................................................

c ...............................................................
c . Now we can load up the solution vectors aa(n) and ba(n)     .
c . and start the LU Decomposition to get t(n) and u(n)         .
c . Note that aa(n) and ba(n) contain the incident angle        .
c . Utilize equations 37-43.                                    .
c ...............................................................
c .............
c For TE case .
c .............
            do 226 n=m,nbg
              i=n-m+1
              c=dzeta_o(n)*psi_o(n)
              c = k1*(c - dpsi_o(n)*zeta_o(n))
              b(i)=aa(n)*c
              b(i+numel)=ba(n)*c
              bd(i)=b(i)
              bd(i+numel)=b(i+numel)
 226        continue
            ii=isiz2
            call ludcmp(matrixlu,numtot,ii,indx,dd)
            call lubksb(matrixlu,numtot,ii,indx,b)
            call mprove(matrix,matrixlu,numtot,ii,indx,bd,b)
            do 227 n=m,nbg
              i=n-m+1
              TintTE(m,n)=b(i)
              UintTE(m,n)=b(i+numel)
 227        continue

c ........................................................
c . This little section here is used to check the ludcmp .
c . subroutine.  It follows from equation 37             .
c ........................................................
c       write(8,*)'m, n, aa(n)*gamma'
c       do 777 n=m,nbg
c          c=k1*(dzeta_o(n)*psi_o(n)-dpsi_o(n)*zeta_o(n))
c          c=aa(n)*c
c          write(8,*)m,n,c
c 777   continue
c       write(8,*)'++++++++++++++++++++++++++++++++++++'
c       do 778 i=1,numel
c          s1=0.0
c          do 779 jj=1,numtot
c               c=matrix(i,jj)*b(jj)
c               s1=s1+c
c 779      continue
c          write(8,*)m,i-1,s1
c 778   continue
c       if(m.eq.17) goto 999

c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c .  Now we must find the Scattering coefficients CscaTE and DscaTE,    .
c .  and from these coeff we can determine the scattering matrices.     .
c .  The equations are  33 and 35 (or 34 and 36).                       .
c .......................................................................
        do 250 n=m,nbg
           sum1=0.0
           sum2=0.0
           do 260 np=m,nbg
              fact1=dzeta2_1(n)+Q_r(np)*dzeta1_1(n)
              fact1=TransAm(np,n)*TintTE(m,np)*fact1
              fact2=dzeta2_1(n)+Q_s(np)*dzeta1_1(n)
              fact2=TransBm(np,n)*UintTE(m,np)*fact2
              sum1=fact1+fact2+sum1
              fact3=zeta2_1(n)+Q_r(np)*zeta1_1(n)
              fact3=TransBm(np,n)*TintTE(m,np)*fact3
              fact4=zeta2_1(n)+Q_s(np)*zeta1_1(n)
              fact4=TransAm(np,n)*UintTE(m,np)*fact4
              sum2=sum2+fact3+fact4
 260       continue
           CscaTE(m,n)=(sum1-aa(n)*dpsi_o(n))/dzeta_o(n)
           DscaTE(m,n)=(sum2-ba(n)*psi_o(n))/zeta_o(n)
c       print*,n,CscaTE(m,n),DscaTE(m,n)
 250    continue

c.................  
c FOR TM CASE!!! .         
c.................
            do 228 n=m,nbg
              i=n-m+1
              c=dzeta_o(n)*psi_o(n)
              c =k1*(c - dpsi_o(n)*zeta_o(n))
              b(i)=ccc*ba(n)*c
              b(i+numel)=ccc*aa(n)*c
              bd(i)=b(i)
              bd(i+numel)=b(i+numel)
 228        continue
            ii=isiz2
            call lubksb(matrixlu,numtot,ii,indx,b)
            call mprove(matrix,matrixlu,numtot,ii,indx,bd,b)
            do 229 n=m,nbg
              i=n-m+1
              TintTM(m,n)=b(i)
              UintTM(m,n)=b(i+numel)
 229        continue

c ........................................................
c . This little section here is used to check the ludcmp .
c . subroutine.  It follows from equation 37             .
c ........................................................
c       write(8,*)'m, n, aaTM(n)*gamma'
c       do 877 n=m,nbg
c          c=k1*(dzeta_o(n)*psi_o(n)-dpsi_o(n)*zeta_o(n))
c          c=ccc*ba(n)*c
c          write(8,*)m,n,c
c 877   continue
c       write(8,*)'++++++++++++++++++++++++++++++++++++'
c       do 878 i=1,numel
c          s1=0.0
c          do 879 jj=1,numtot
c               c=matrix(i,jj)*b(jj)
c               s1=s1+c
c 879      continue
c          write(8,*)m,i-1,s1
c 878   continue
c       if(m.eq.17) goto 999

c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c .  Now we must find the Scattering coefficients CscaTM and DscaTM,    .
c .  and from these coeff we can determine the scattering matrices.     .
c .  The equations are gotten from 33 and 35                            .
c .......................................................................
        do 251 n=m,nbg
           sum1=0.0
           sum2=0.0
           do 261 np=m,nbg
              fact1=dzeta2_1(n)+Q_r(np)*dzeta1_1(n)
              fact1=TransAm(np,n)*TintTM(m,np)*fact1
              fact2=dzeta2_1(n)+Q_s(np)*dzeta1_1(n)
              fact2=TransBm(np,n)*UintTM(m,np)*fact2
              sum1=fact1+fact2+sum1
              fact3=zeta2_1(n)+Q_r(np)*zeta1_1(n)
              fact3=TransBm(np,n)*TintTM(m,np)*fact3
              fact4=zeta2_1(n)+Q_s(np)*zeta1_1(n)
              fact4=TransAm(np,n)*UintTM(m,np)*fact4
              sum2=sum2+fact3+fact4
 261       continue
           CscaTM(m,n)=dpsi_o(n)*ccc*ba(n)
           CscaTM(m,n)=(sum1-CscaTM(m,n))/dzeta_o(n)
           DscaTM(m,n)=ccc*aa(n)*psi_o(n)
           DscaTM(m,n)=(sum2-DscaTM(m,n))/zeta_o(n)
c       print*,n,CscaTM(m,n),DscaTM(m,n)
 251    continue

c .......................................................................
c . We can now calculate the Efficiencies (Qext, Qsca, Qabs) by using   .
c . the equations 58 - 60.                                              .
c .......................................................................
           do 510 n=1,nbg

              tau=CscaTE(m,n)*dconjg(CscaTE(m,n))
              tau=tau+DscaTE(m,n)*dconjg(DscaTE(m,n))
              pie=CscaTM(m,n)*dconjg(CscaTM(m,n))
              pie=pie+DscaTM(m,n)*dconjg(DscaTM(m,n))
              x=n*(n+1.)*(tau+pie)

              s1=CscaTE(m,n)*dconjg(aa(n))
              s1=s1+DscaTE(m,n)*dconjg(ba(n))
              s2=CscaTM(m,n)*dconjg(ccc*ba(n))
              s2=s2+DscaTM(m,n)*dconjg(ccc*aa(n))
              s3=n*(n+1.)*(s2+s1)
              if(m.eq.0) then
                 x=2.0*x
                 s3=-2.0*s3
              else
                 x=4.0*x
                 s3=-4.0*s3
              endif

              qsca=qsca +x
              qext=qext +dreal(s3)
 510       continue
c ........................................................................
c .  That finishes one loop in m, now we move to the next m and repeat  .
c .  the same procedure all over again, storing our internal results    .
c .  into T_te, U_te, T_tm, U_tm, and storing the scattering results    .
c .  into C_te, D_te, C_tm, D_tm.                                       .
c .......................................................................
 299   continue
c............................................
c DONE!!  The sum over m is now complete!!! .
c............................................
c        print*,'Cext =',qext*wavel**2/(4.*pi)
c        print*,'Cext=',qext/cdabs(k0)**2

        qext=qext/cdabs(x0)**2
        qsca=qsca/cdabs(x0)**2
        qabs=qext-qsca
c       write(8,*) real(distance),real(qext),real(qsca),real(qabs)
        
c        print*,'Qext, Qsca, Qabs=',qext,qsca,qabs
c ************************************************************
c COMMENT THIS BLOCK OUT IF YOU DON'T WANT TO FIND
c THE ASYMMETRY PARAMETER.  THIS FIND_G SUBROUTINE
c IS VERY TIME CONSUMING (Use equation 61)
        ii=isize1
        alpha=inc_ang
        gg=find_g(nbg,ii,alpha,CscaTE,DscaTE,CscaTM,DscaTM)
        gg=gg/x0**2/qsca
c        print*, 'asym parameter =',gg
c ************************************************************
        write(8,*) 'Output file:'
        write(8,*) ' '
        write(8,*) '   distance    Qext       Qsca      Qabs          g'
        write(8,*) real(distance),real(qext),real(qsca),real(qabs)
     +            ,real(gg)
        write(8,*) ' '
        qext=0.0
        qsca=0.0
        qabs=0.0
c        print*, '********************************************'
c        print*, 'Done with the rough stuff, now to find soln'
c        print*, '********************************************'

c .....................................................................
c .  Here's the meat of the problem.  We will calculate the amplitude .
c . scattering matrices for the fields that we've found.  The angle   .
c . theta is our observation angle.  Num is the number of angles we   .
c . want in determining the scattering intensity.  The rest of this   .
c . should look almost like the Mie Code found in Bohren and Huffman  .
c . We have here 3 nested loops.  The first one is to determine the   .
c . angles we want to look at, the second one is the sum over m, and  .
c . the third one sums over n.                                        .
c . If we want to have the observation angle's resolution of 1 degree .
c . then set num=360, if we want finer resolution, let num be a bigger.
c . number such as 720 (to have resolution of 0.5degrees)             .
c .....................................................................
        write(8,*) 'angle     S11         S12         S33         S34'
 
        phi_o=dfloat(phi*pi/180.0)
      i=0
      d_ang=360.0/num
         b_angle=inc_ang+180.
         f_angle=inc_ang
        do 399 j=1,num+1
         angle=d_ang*(j-1)
         angledd=angle

c       if((angle.eq.b_angle).or.(angle.eq.f_angle)) then
c       if((angle.le.180.0).and.(angle.ge.0.0)) then
c        print*,'Incident angle is',inc_ang
c        print*,'Obs ANGLE IS ',angle
         if(angle.gt.180.0) then
                angle=360.0-angle
                phi=pi + phi_o
         else
                phi=phi_o
         endif
            i=i+1
            s1TE(i)=0.
            s3TE(i)=0.
            s2TM(i)=0.
            s4TM(i)=0.

            theta=angle*pi/180.
            x=dcos(theta)

            do 330 m=0,nbg
c       ...........................................................
c       .  The next few lines will calc the Legendre polynomial   .
c       .  at the angle theta                                     .
c       ...........................................................
                  ii=nbg+1
                  mmm=m
                  call nplgndr(mmm,ii,isize1,legpol2,x)
                  mmm=m+1
                  call nplgndr(mmm,ii,isize1,legpol1,x)
                  if (m.eq.0) then
                        do 305 jj=-2,isize1
                              legpol(jj)=-legpol1(jj)
 305                    continue
                  else
                        mmm=m-1
                        call nplgndr(mmm,ii,isize1,legpol,x)
                  endif

                  do 310 n=m,nbg      
c       .........................................................
c       .  Calculate the corresponding pie and tau              .
c       .........................................................
                    tau=sqrt((n+m+1.)*(n-m))*legpol1(n)
                    pie=tau
                    tau=-(tau-sqrt((n+m)*(n-m+1.))*legpol(n))/2.
                    if (dabs(x).gt.0.5) then
                      pie=pie+sqrt((n+m)*(n-m+1.))*legpol(n)
                      pie=pie*.5/x
                    else
                      pie=m*legpol2(n)/dsin(theta)
                    endif
c .......................................................
c .Done with finding Tau and Pie!                       .
c .Now to find the scattering amplitude S1,S2,S3,and S4 .
c .......................................................

                    ifact=(-ccc)**n
                    if(m.eq.0) then
                      fact1=DscaTE(0,n)*pie+CscaTE(0,n)*tau
                      fact2=CscaTM(0,n)*pie+DscaTM(0,n)*tau
                      fact3=CscaTE(0,n)*pie+DscaTE(0,n)*tau
                      fact4=DscaTM(0,n)*pie+CscaTM(0,n)*tau
                      s1=ifact*fact1
                      s2=ifact*fact2
                      s3=ifact*fact3
                      s4=ifact*fact4
                    else
                      fact1=DscaTE(m,n)*pie+CscaTE(m,n)*tau
                      fact1=fact1*2.0*dcos(m*phi)
                      fact2=CscaTM(m,n)*pie+DscaTM(m,n)*tau
                      fact2=fact2*2.0*dcos(m*phi)
                      fact3=CscaTE(m,n)*pie+DscaTE(m,n)*tau
                      fact3=fact3*2.0*ccc*dsin(m*phi)
                      fact4=DscaTM(m,n)*pie+CscaTM(m,n)*tau
                      fact4=fact4*2.0*ccc*dsin(m*phi)
                      s1=ifact*fact1
                      s2=ifact*fact2
                      s3=ifact*fact3
                      s4=ifact*fact4
                    endif
                    s1TE(i)=s1TE(i)+s1 
                    s2TM(i)=s2TM(i)+s2
                    s3TE(i)=s3TE(i)+s3
                    s4TM(i)=s4TM(i)+s4
 310              continue
c ..............................................
c . Finished summing over n, now need to        .
c . sum over m in order to have the complete    .
c . solution for the Scat Amp Matrix            .
c ..............................................
 330        continue

                s1TE(i)=s1TE(i)
                s2TM(i)=-ccc*s2TM(i)
                s3TE(i)=-ccc*s3TE(i)
                s4TM(i)=s4TM(i)
c ..............................................
c . Done summing over m.  Now we have one       .
c . complete set of scat amp S(i)               .
c ..............................................
c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c .  Lastly, we can now use the scattering amplitude matrix     .
c .  and solve for the Mueller Matrix elements.  Once we have   .
c .  the MM, we have all the informations we need.              .
c +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            do 410 jj=1,4
            mm(1,jj)=0.0
            mm(2,jj)=0.0
            mm(3,jj)=0.0
            mm(4,jj)=0.0
 410    continue
            fact1=s1TE(i)*dconjg(s1TE(i))/2.
            fact2=s2TM(i)*dconjg(s2TM(i))/2.
            fact3=s3TE(i)*dconjg(s3TE(i))/2.
            fact4=s4TM(i)*dconjg(s4TM(i))/2.
            c=s2TM(i)*dconjg(s3TE(i))
            cc=s1TE(i)*dconjg(s4TM(i))
            mm(1,1)=dreal(fact1+fact2+fact3+fact4)
            mm(1,2)=dreal(fact2-fact1-fact3+fact4)/mm(1,1)
            mm(1,3)=dreal(c + cc)/mm(1,1)
            mm(1,4)=dimag(c -cc)/mm(1,1)

            mm(2,1)=dreal(fact2-fact1+fact3-fact4)
            mm(2,2)=dreal(fact2+fact1-fact3-fact4)/mm(1,1)
            mm(2,3)=dreal(c -cc)/mm(1,1)
            mm(2,4)=dimag(c+cc)/mm(1,1)

            fact1=s2TM(i)*dconjg(s4TM(i))
            fact2=s1TE(i)*dconjg(s3TE(i))
            fact3=s1TE(i)*dconjg(s2TM(i))
            fact4=s3TE(i)*dconjg(s4TM(i))
            c=s2TM(i)*dconjg(s1TE(i))
            cc=s4TM(i)*dconjg(s3TE(i))
            mm(3,1)=dreal(fact1+fact2)
            mm(3,2)=dreal(fact1-fact2)/mm(1,1)
            mm(3,3)=dreal(fact3+fact4)/mm(1,1)
            mm(3,4)=dimag(c + cc)/mm(1,1)

            fact1=s4TM(i)*dconjg(s2TM(i))
            fact3=s1TE(i)*dconjg(s2TM(i))
            mm(4,1)=dimag(fact1+fact2)
            mm(4,2)=dimag(fact1-fact2)/mm(1,1)
            mm(4,3)=dimag(fact3-fact4)/mm(1,1)
            mm(4,4)=dreal(fact3-fact4)/mm(1,1)
 391    format(i4,'    ',f10.5,3('    ',f8.5))
        write(8,391)int(angledd),real(mm(1,1)),real(mm(1,2)),
     +   real(mm(3,3)),real(mm(3,4))
c        print*,real(angledd),real(mm(1,1))
c       write(8,*)int(angle),real(mm(1,1)),real(mm(1,2)),
c     + real(mm(3,3)),real(mm(3,4))
c       print*,'f_angle  and bangle=',f_angle,b_angle
c       if(f_angle.eq.0.0) write(9,*)real(distance),real(mm(1,1))
c       if(b_angle.eq.180.0) write(10,*)real(distance),real(mm(1,1))
c       if (angle.eq.90.0) write(11,*)real(inc_ang),real(mm(1,1))

c        endif
c ---------------------------------------------------------------------
c  This endif statement lets us select the angles we want to look at  .
c ---------------------------------------------------------------------
 399  continue
c       .........................................................
c       .  Done with calculating the amplitude coefficients     .
c       .........................................................

c        print*, ' '
 999    close(4)
       close(8)
       stop
       end

